import { EntityHealableComponent, EntityHealthComponent, system, world } from '@minecraft/server';
import { WeaponLibrary } from './libs/guns';
import { support } from './libs/support/support';


const weapons = {
    "allanz:sword_gold": { damage: 5, fire: true, particle: 'allanz:gold_aura', knockback: true, effects: ['invisibility', 'hunger', 'weakness'] },
    // "allanz:sword_prata": { damage: 5, fire: false, particle: 'allanz:prata_aura', knockback: true, effects: ['invisibility', 'hunger', 'weakness'],  },
};

const weaponLib = new WeaponLibrary(weapons);


world.afterEvents.itemStartUse.subscribe((event) => {
    weaponLib.handleItemStartUse(event);
});

world.afterEvents.itemStopUse.subscribe((event) => {
    weaponLib.handleItemStopUse();
});


world.afterEvents.entityHitEntity.subscribe((event) => {
    const { hitEntity, damagingEntity } = event;
    try {
        if (damagingEntity.typeId.includes('player') && support.getItem(damagingEntity, { slot: "mainhand" }).typeId == 'allanz:sword_prata') {
            hitEntity.addEffect('poison', 100, { amplifier: 5, showParticles: false });
            hitEntity.addEffect('nausea', 100, { amplifier: 5, showParticles: false });
            hitEntity.addEffect('weakness', 100, { amplifier: 5, showParticles: false });
            hitEntity.dimension.spawnParticle('allanz:prata_aura', hitEntity.location)
        } else if (damagingEntity.typeId.includes('player') && support.getItem(damagingEntity, { slot: "mainhand" }).typeId == 'allanz:sword_esmerald') {
            /**
             * @type {EntityHealthComponent} 
             */
            const damagingHealth = damagingEntity.getComponent('health');
            /**
            * @type {EntityHealthComponent} 
            */
            const hitHealth = hitEntity.getComponent('health');
            damagingHealth.setCurrentValue(damagingHealth.currentValue + ((hitHealth.currentValue - hitHealth.effectiveMax < 0 && hitHealth.effectiveMax != -hitHealth.effectiveMax ? hitHealth.currentValue - hitHealth.effectiveMax : 0) * -1) * 0.25);
            damagingEntity.dimension.spawnParticle('allanz:healing', damagingEntity.location);
        } else if (damagingEntity.typeId.includes('player') && support.getItem(damagingEntity, { slot: "mainhand" }).typeId == 'allanz:sword_moon') {
            if (damagingEntity.isSneaking) {
                hitEntity.dimension.spawnParticle('allanz:lunar', hitEntity.location);
                hitEntity.applyKnockback(0, 0, 0, 1.5);
            } else {
                hitEntity.dimension.spawnParticle('allanz:lunar', hitEntity.location);
                hitEntity.addEffect('slowness', 100, { amplifier: 5, showParticles: false });
            }
        }
    } catch (error) {
        return
    }

})

world.afterEvents.itemUse.subscribe((event) => {
    const { source, itemStack } = event;
    if (itemStack.typeId == 'allanz:sword_diamond') {
        source.dimension.spawnParticle('allanz:light_diamond', source.location);
        source.addEffect('invisibility', 100, { showParticles: false });
        source.playSound('random.explose', { location: source.location });
        source.dimension.getEntities({ maxDistance: 10, minDistance: 0, location: source.location }).forEach((victim) => {
            if (victim.id != source.id) {
                victim.addEffect('nausea', 100, { amplifier: 5, showParticles: true });
                victim.addEffect('weakness', 100, { amplifier: 5, showParticles: true });
                victim.addEffect('slowness', 100, { amplifier: 5, showParticles: true });
            }
        })
    }
})

system.afterEvents.scriptEventReceive.subscribe((event) => {
    const { sourceEntity, id, message } = event;
})